#<profile4xi profile = "03" />
#<program4xi program = "connection4xi.py">
#<abstract4xi abstract = "providing class Connection4Xi"/>
#<refer4xi reference = "connection.py"/> 
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'Connection4Xi']
#"
#/>
#

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import io
import os
import sys
import pickle
import select
import socket
import struct
import errno
import time
import tempfile
import itertools  
try:
    import _winapi
    from _winapi import WAIT_OBJECT_0, WAIT_TIMEOUT, INFINITE
except ImportError:
    if sys.platform == 'win32':
        raise
    _winapi = None               
   
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
from multiprocessing import connection
from multiprocessing.connection import Connection
from multiprocessing import reduction
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from forkingpickler4xi import ForkingPickler4Xi
from current_process4xi import get_authkey_current_process4xi
from register_after_fork4xi import register_after_fork4xi
from sub_warning4xi import sub_warning4xi
from timeout4xi import omega2timeout4xi
from print4xi import print4xi
from time4xi import time4xi    
from wait4xi import wait4xi
from _connectionbase4xi import _ConnectionBase4Xi
#"
#/>

# =================================================================================================
#<class4xi class = "Connection4Xi" base = "object" child = "None" synopsis = "Connection4Xi class" usage = "
#   connection.py
#       if sys.platform != 'win32':
#           Pipe(duplex=True)
#               if duplex:
#                   s1, s2 = socket.socketpair()
#                   s1.setblocking(True)
#                   s2.setblocking(True)
#                   c1 = Connection(s1.detach())
#                   c2 = Connection(s2.detach())
#               else:
#                   fd1, fd2 = os.pipe()
#                   c1 = Connection(fd1, writable=False)
#                   c2 = Connection(fd2, readable=False)
#   pipe4xi.py
#       if sys.platform != get_windows_system4xi(): # dragonized
#           def Pipe4Xi(duplex=True):
#               if duplex: # <-- Python 3.2 version c1_2xi = _multiprocessing.Connection(os.dup(s12xi.fileno()))
#                   s1, s2 = socket.socketpair()
#                   s1.setblocking(True)
#                   s2.setblocking(True)
#                   c1 = Connection4Xi(s1.detach())
#                   c2 = Connection4Xi(s2.detach())
#               else:
#                   fd1, fd2 = os.pipe()
#                   c1 = Connection4Xi(fd1, writable=False)
#                   c2 = Connection4Xi(fd2, readable=False)               
#>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# =================================================================================================
class Connection4Xi(_ConnectionBase4Xi):
    '''
    Connection4Xi class based on an arbitrary file descriptor (Unix only), or
    a socket handle (Windows).
    '''

    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = ('sense', 'reason', 'ace')          
    #"
    #/>    
     #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables set depending on OS
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    if _winapi:
        # --------------------------------------------------------------------------------------------------------
        #<function4xi function = _close usage = "closes socket for Connection4Xi class objects">
        #<note4xi note = [Rel 7.21]""/>
        #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
        # --------------------------------------------------------------------------------------------------------
        def _close(self, _close=_multiprocessing.closesocket):
            class2print = "Connection4Xi"
            method2print = "_close"
            
            message2print = "{(Step 1/1a) started" + "; _close-of-Connection4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #    
            
            message2print = "{(Step 1/1b) passed" + "; _close-passed-to-_close-of-Connection4Xi: " + str(_close) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
            
            message2print = "{(Step 1/1c) passing" + "; handle-passing-to-_close-by-_close-of-Connection4Xi: " + str(self._handle) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            # 
            
            _close(self._handle)
            
            message2print = "{(Step 1/1d) performed" + "; _close-performed-by-_close-of-Connection4Xi to: " + str(_close) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        #</function4xi> 
            
        _write = _multiprocessing.send
         
        class2print = "Connection4Xi"
        method2print = "share"
           
        message2print = "{(Step 1/1e) set" + "; _write-set-by-_close-of-Connection4Xi to: " + str(_write) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
            
        _read = _multiprocessing.recv
            
        message2print = "{(Step 1/1f) set" + "; _read-set-by-_close-of-Connection4Xi to: " + str(_read) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
            
                   
    else:
        # ---------------------------------------------------------------------------------------------
        #<function4xi function = "_close-of-Connection4Xi" usage = "closes socket for Connection4Xi class objects">
        #<note4xi note = [Rel 7.21]""/>
        #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
        # ---------------------------------------------------------------------------------------------
        def _close(self, _close=os.close):
            class2print = "Connection4Xi"
            method2print = "_close"
            
            message2print = "{(Step 1/1a) started" + "; _close-of-Connection4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #    
            
            message2print = "{(Step 1/1b) passed" + "; _close-passed-to-_close-of-Connection4Xi: " + str(_close) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
                    
            _close(self._handle)
            
            message2print = "{(Step 1/1c) performed" + "; _close-performed-by-_close-of-Connection4Xi: " + str(_close) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        class2print = "Connection4Xi"
        method2print = "share"            
            
        _write = os.write
        message2print = "{(Step 1/1e) set" + "; _write-set-by-_close-of-Connection4Xi to: " + str(_write) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #         
        
        _read = os.read 
        message2print = "{(Step 1/1f) set" + "; _read-set-by-_close-of-Connection4Xi to: " + str(_read) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #         
               
        #</function4xi>              
    #"
    #/>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-Connection4Xi" usage = "initializes Connection4Xi with (self)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self):
        '''
        initializes Connection4Xi
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/1)            
        # -----------------------------------------------------------------------------------------
        class2print = "Connection4Xi"
        method2print = "__init__"
        
        message2print = "{(Step 1/1a) started" + "; init-of-Connection4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
          
        message2print = "{(Step 1/1b) activated" + "; connection2xi-of-Connection4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #                 
        
        message2print = "{(Step 1/1c) inherited" + "; _ConnectionBase4Xi-inherited-by-connection2xi-of-Connection4Xi: " + str(_ConnectionBase4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #    
                 
        message2print = "{(Step 1/1d) returning" + "; init-of-Connection4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #                        
                
    #</methodize4xi>
            
   

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "_send-of-Connection4Xi" usage = "sends for Connection4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def _send(self, buf, write=_write):
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)      
        # -----------------------------------------------------------------------------------------
        class2print = "Connection4Xi"
        method2print = "_send"
                       
        message2print = "{(Step 1/3a) started" + "; _send-of-Connection4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #    
        
        message2print = "{(Step 1/3b) passed" + "; buf-passed-to-_send-of-Connection4Xi: " + str(buf) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        message2print = "{(Step 1/3c) passed" + "; write-passed-to-_send-of-Connection4Xi: " + str(write) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                
        remaining2xi = len(buf)
        message2print = "{(Step 1/3d) set" + "; remaining2xi-set-by-_send-of-Connection4Xi to: " + str(remaining2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/3)      
        # -----------------------------------------------------------------------------------------        
        while True:
            n2xi = write(self._handle, buf)
            message2print = "{(Step 2/3a) got" + "; n2xi-got-by-_send-of-Connection4Xi: " + str(n2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        
            remaining2xi -= n2xi
            message2print = "{(Step 2/3b) set" + "; remaining2xi-set-by-_send-of-Connection4Xi to: " + str(remaining2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            if remaining2xi == 0:
                message2print = "{(Step 2/3c) got" + "; remaining2xi-got-by-_send-of-Connection4Xi to: " + str(remaining2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                break
                
            buf = buf[n2xi:] 
            message2print = "{(Step 2/3d) set" + "; buf-set-by-_send-of-Connection4Xi to: " + str(buf) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/3)      
        # -----------------------------------------------------------------------------------------                                      
        message2print = "{(Step 3/3) returning" + "; _send-of-Connection4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #
           
    #</methodize4xi>                
        
        # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "_recv-of-Connection4Xi" usage = "receives for Connection4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def _recv(self, size, read=_read):
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4)      
        # -----------------------------------------------------------------------------------------
        class2print = "Connection4Xi"
        method2print = "_recv"
                       
        message2print = "{(Step 1/4a) started" + "; _recv-of-Connection4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #    
        
        message2print = "{(Step 1/4b) passed" + "; size-passed-to-_recv-of-Connection4Xi: " + str(size) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        message2print = "{(Step 1/4c) passed" + "; read-passed-to-_recv-of-Connection4Xi: " + str(read) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/4)      
        # -----------------------------------------------------------------------------------------                
        buf2xi = io.BytesIO()
        message2print = "{(Step 2/4a) set" + "; buf2xi-set-by-_recv-of-Connection4Xi to: " + str(buf2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        handle2xi = self._handle
        message2print = "{(Step 2/4b) set" + "; handle2xi-set-by-_recv-of-Connection4Xi to: " + str(handle2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        
        remaining2xi = size
        message2print = "{(Step 2/4c) set" + "; remaining2xi-set-by-_recv-of-Connection4Xi to: " + str(remaining2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/4)      
        # -----------------------------------------------------------------------------------------        
        while remaining2xi > 0:
            chunk2xi = read(handle2xi, remaining2xi)
            message2print = "{(Step 3/4a) got" + "; chunk2xi-got-by-_recv-of-Connection4Xi to: " + str(chunk2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            n2xi = len(chunk2xi)
            message2print = "{(Step 3/4b) set" + "; n2xi-set-by-_recv-of-Connection4Xi to: " + str(n2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            if n2xi == 0:
                if remaining2xi == size:
                    message2print = "{(Step 3/4z11) failed" + "; remaining2xi-failed-by-_recv-of-Connection4Xi: " + str(remaining2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    message2print = "{(Step 3/4z12) failed" + "; size-failed-by-_recv-of-Connection4Xi: " + str(size) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    message2print = "{(Step 3/4z13) failed" + "; n2xi-failed-by-_recv-of-Connection4Xi: " + str(n2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    message2print = "{(Step 3/4z14) failed" + "; chunk2xi-failed-by-_recv-of-Connection4Xi: " + str(chunk2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    raise EOFError
                else:
                    message2print = "{(Step 3/4z21) failed" + "; remaining2xi-failed-by-_recv-of-Connection4Xi: " + str(remaining2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    message2print = "{(Step 3/4z22) failed" + "; size-failed-by-_recv-of-Connection4Xi: " + str(size) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    message2print = "{(Step 3/4z23) failed" + "; n2xi-failed-by-_recv-of-Connection4Xi: " + str(n2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    message2print = "{(Step 3/4z24) failed" + "; chunk2xi-failed-by-_recv-of-Connection4Xi: " + str(chunk2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    raise IOError("_recv-of-Connection4Xi got end of file during message")
             
            message2print = "{(Step 3/4c) passing" + "; chunk2xi-passing-to-buf2xi-write-by-_recv-of-Connection4Xi: " + str(chunk2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            #         
                           
            buf2xi.write(chunk2xi)
            message2print = "{(Step 3/4d) performed" + "; write-performed-by-_recv-of-Connection4Xi: " + str(buf2xi.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            remaining2xi -= n2xi
            message2print = "{(Step 3/4e) calculated" + "; remaining2xi-calculated-by-_recv-of-Connection4Xi: " + str(remaining2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/4)      
        # -----------------------------------------------------------------------------------------                                      
        message2print = "{(Step 4/4) returning" + "; buf2xi-returning-by-_recv-of-Connection4Xi: " + str(buf2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        return buf2xi
           
    #</methodize4xi>    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "_send-of-Connection4Xi" usage = "sends for Connection4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def _send_bytes(self, buf):
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4)      
        # -----------------------------------------------------------------------------------------
        class2print = "Connection4Xi"
        method2print = "_send"
                       
        message2print = "{(Step 1/4a) started" + "; _send-of-Connection4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #    
        
        message2print = "{(Step 1/4b) passed" + "; buf-passed-to-_send-of-Connection4Xi: " + str(buf) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/4)      
        # -----------------------------------------------------------------------------------------                
        n2xi = len(buf)
        message2print = "{(Step 2/4a) set" + "; n2xi-set-by-_send-of-Connection4Xi to: " + str(n2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/4)      
        # -----------------------------------------------------------------------------------------        
        try:
            self._send(struct.pack("!i", n2xi))
            message2print = "{(Step 3/4a) performed" + "; send-performed-by-_send-of-Connection4Xi to: " + str(self._send) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            if n2xi > 0:
                # The condition is necessary to avoid "broken pipe" errors
                # when sending a 0-length buffer if the other end closed the pipe.
                message2print = "{(Step 3/4b) passing" + "; buf-passing-to-send-by-_send-of-Connection4Xi: " + str(buf) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                #  
                
                self._send(buf)
                
                message2print = "{(Step 3/4c) performed" + "; send-performed-by-_send-of-Connection4Xi to: " + str(self._send) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            
        except:
            message2print = "{(Step 3/4z1) failed" + "; buf-failed-by-_send-of-Connection4Xi to: " + str(buf) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            message2print = "{(Step 3/4z1) failed" + "; n2xi-failed-by-_send-of-Connection4Xi to: " + str(n2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/4)      
        # -----------------------------------------------------------------------------------------                                      
        message2print = "{(Step 1/1d) returning" + "; _send-of-Connection4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #   
        return
           
    #</methodize4xi>
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "_receive-of-Connection4Xi" usage = "receives for Connection4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def _recv_bytes(self, maxsize=None):
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4)      
        # -----------------------------------------------------------------------------------------
        class2print = "Connection4Xi"
        method2print = "_receive"
                       
        message2print = "{(Step 1/4a) started" + "; _receive-of-Connection4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #    
        
        message2print = "{(Step 1/4b) passed" + "; maxsize-passed-to-_receive-of-Connection4Xi: " + str(maxsize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/4)      
        # -----------------------------------------------------------------------------------------                
        buf2xi = None
        try:
            buf2xi = self._recv(4)
        
            message2print = "{(Step 2/4a) got" + "; buf2xi-got-by-_receive-of-Connection4Xi: " + str(buf2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        except:
            message2print = "{(Step 2/4z) failed" + "; buf2xi-failed-by-_receive-of-Connection4Xi: " + str(buf2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/4)      
        # -----------------------------------------------------------------------------------------        
        size2xi = 0
        try:
            size2xi, = struct.unpack("!i", buf2xi.getvalue())
            message2print = "{(Step 3/4a) " + "; size2xi-got-by-_receive-of-Connection4Xi: " + str(size2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            if maxsize is not None and size2xi > maxsize:
                message2print = "{(Step 3/4z1) warned" + "; maxsize-warned-by-_receive-of-Connection4Xi: " + str(maxsize) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                message2print = "{(Step 3/4z2) warned" + "; size2xi-warned-by-_receive-of-Connection4Xi: " + str(size2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 

                message2print = "{(Step 3/4z3) warned" + "; buf2xi-warned-by-_receive-of-Connection4Xi: " + str(buf2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 

                # 
                size2xi = maxsize                                                                                   # dragonized to receive maximum
            
                message2print = "{(Step 3/4z4) set" + "; size2xi-set-by-_receive-of-Connection4Xi to: " + str(size2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
        except:
            message2print = "{(Step 3/4z1) failed" + "; maxsize-failed-by-_receive-of-Connection4Xi to: " + str(maxsize) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            message2print = "{(Step 3/4z2) failed" + "; n2xi-failed-by-_receive-of-Connection4Xi to: " + str(n2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            message2print = "{(Step 3/4z3) failed" + "; buf2xi-failed-by-_receive-of-Connection4Xi to: " + str(buf2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/4)      
        # -----------------------------------------------------------------------------------------                                      
        message2print = "{(Step 4/4) returning" + "; size2xi-returning-by-_receive-of-Connection4Xi: " + str(size2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        return self._recv(size2xi)
           
    #</methodize4xi>    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "_poll-of-Connection4Xi" usage = "polls for Connection4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def _poll(self, timeout):
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)      
        # -----------------------------------------------------------------------------------------
        class2print = "Connection4Xi"
        method2print = "_poll"
                       
        message2print = "{(Step 1/3a) started" + "; _poll-of-Connection4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #    
        
        message2print = "{(Step 1/3b) passed" + "; timeout-passed-to-_poll-of-Connection4Xi: " + str(timeout) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/3)      
        # -----------------------------------------------------------------------------------------                
        r2xi = None
        try:
            r2xi = wait4xi([self._handle], timeout)
            message2print = "{(Step 2/3a) got" + "; r2xi-got-by-_poll-of-Connection4Xi: " + str(r2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        except:
            message2print = "{(Step 2/3z) failed" + "; r2xi-failed-by-_poll-of-Connection4Xi: " + str(r2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/3)      
        # -----------------------------------------------------------------------------------------                                      
        message2print = "{(Step 3/3) returning" + "; r2xi-returning-by-_poll-of-Connection4Xi: " + str(r2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        return bool(r2xi)
           
    #</methodize4xi>                                                                                                                                      
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-Connection4Xi" usage = "senses Connection4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses Connection4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
        
        class2print = 'Connection4Xi'
        method2print = 'sense'
        
           
        message2print = "{(Step 1/5) started" + "; sense-of-Connection4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------  
        message2print = "{(Step 2/5) sensing" + "; Connection4Xi-sensing-by-sense-of-Connection4Xi: " + str(Connection4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                  
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3/5) got" + "; knowledge-base2xi-got-by-sense-of-Connection4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/5) add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
    
        
        
      
        message2print = "{(Step 4/5) sensed" + "; Connection4Xi-sensed-by-sense-of-Connection4Xi: " + str(Connection4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------         
        message2print = "{(Step 5/5) returning" + "; knowledge-base2xi-returning-by-sense-of-Connection4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
          
        return kb2xi
        
    #</methodize4xi>
      

        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-Connection4Xi" usage = "reasons Connection4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons Connection4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
             
        
        class2print = 'Connection4Xi'
        method2print = 'reason'

           
        message2print = "{(Step 1/5) started" + "; reason-of-Connection4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2/5) reasoning" + "; Connection4Xi-reasoning-by-reason-of-Connection4Xi: " + str(Connection4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
            
        
        # -----------------------------------------------------------------------------------------
        #  (Step 3/5) get reasoned knowledge for this object -  Non Windows
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        message2print = "{(Step 3/5) got" + "; knowledge-base4xi-got-by-by-reason-of-Connection4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            
        
        # -----------------------------------------------------------------------------------------
        #  (Step 4/5) add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        
        message2print = "{(Step 4/5) reasoning" + "; Connection4Xi-reasoning-by-reason-of-Connection4Xi: " + str(Connection4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 5/5) returning" + "; knowledge-base4xi-returning-by-reason-of-Connection4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                          
          
        return kb4xi
        
    #</methodize4xi>
         

    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-Connection4Xi" usage = "aces Connection4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces Connection4Xi class objects 
        '''
          
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
            
        
        class2print = 'Connection4Xi'
        method2print = 'ace'
        
           
        message2print = "{(Step 1/5) started" + "; ace-of-Connection4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2/5) acing" + "; Connection4Xi-acing-by-ace-of-Connection4Xi: " + str(Connection4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
            
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/5) get aced knowledge for this object -  Non Windows
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3/5)got" + "; knowledge-base8xi-got-by-ace-of-Connection4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            


        # -----------------------------------------------------------------------------------------
        # (Step 4/5) add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{(Step 4/5) acing" + "; Connection4Xi-acing-by-ace-of-Connection4Xi: " + str(Connection4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 5/5) returning" + "; knowledge-base8xi-returning-by-ace-of-Connection4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
        
        return kb8xi
    #</methodize4xi>
#</class4xi>

# ====================================================================================================================
# REGISTER
#====================================================================================================================
if sys.platform == 'win32':
    ForkingPickler4Xi.register(socket.socket, reduction.reduce_socket)
    ForkingPickler4Xi.register(Connection4Xi, reduction.reduce_connection)
 
else:
    ForkingPickler4Xi.register(socket.socket, reduction.reduce_socket)
    ForkingPickler4Xi.register(Connection4Xi, reduction.reduce_connection)


#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of connection4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

